<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
**/

namespace MAM\Daemon\Api;

class HttpClient extends TcpClient implements ApiClientInterface {
    private $request;
    private $header;
    private $body;

    /**
     * @param $socket resource
     */
    public function __construct($socket, $tcp_server) {
        parent::__construct($socket, $tcp_server);
    }

    private function getUserAndPassword() {
        if (!isset($this->header['authorization'])) {
            return false;
        }

        list($auth_type, $base64) = explode(' ', $this->header['authorization'], 2);

        if ($auth_type !== 'Basic') {
            return false;
        }

        list($user, $password) = explode(':', base64_decode($base64, true), 2);
        return ['user' => $user, 'password' => $password];
    }

    public function getUser() {
        $user_and_password = $this->getUserAndPassword();
        if ($user_and_password === false) {
            return false;
        }
        return $user_and_password['user'];
    }

    public function getPassword() {
        $user_and_password = $this->getUserAndPassword();
        if ($user_and_password === false) {
            return false;
        }
        return $user_and_password['password'];
    }

    public function requestAuth() {
        $this->writetoSocket("HTTP/1.1 401 Unauthorized\r\n"
            . "WWW-Authenticate: Basic\r\n"
            . "Connection: close\r\n"
            . "Content-Length: 25\r\n"
            . "Content-Type: text/plain\r\n"
            . "Access-Control-Allow-Origin: *\r\n"
            . "\r\n"
            . "HTTP/1.1 401 Unauthorized");
    }

    public function generateResponse($response_body) {
        return "HTTP/1.1 200 OK\r\n"
            . "Connection: close\r\n"
            . "Content-Length: " . strlen($response_body) . "\r\n"
            . "Content-Type: text/plain\r\n"
            . "Access-Control-Allow-Origin: *\r\n"
            . "\r\n"
            . $response_body;
    }

    public function readRequest() {
        $offset = 0; //will be changed (call by reference)

        $this->request = $this->readRequestLine($offset);
        if ($this->request === false) {
            return false;
        }

        if ($this->request['method'] === 'OPTIONS') {
            $this->writeToSocket("HTTP/1.1 200 OK\r\n"
                . "Access-Control-Allow-Origin: *\r\n"
                . "Access-Control-Max-Age: 3628800\r\n"
                . "Access-Control-Allow-Methods: PUT, DELETE, POST, GET\r\n"
                . "Access-Control-Allow-Headers: authorization\r\n"
                . "Connection: close\r\n"
                . "Content-Length: 0\r\n"
                . "\r\n");
            $this->disconnectClient();
            return false;
        }


        $this->header = $this->readHeaderFields($offset);
        if ($this->header === false) {
            return false;
        }

        $this->body = '';
        if ($this->request['method'] === 'POST' || $this->request['method'] === 'PUT') {
            $this->body = $this->readBody($offset);
            if ($this->body === false) {
                return false;
            }
        }

        return true;
    }

    public function executeRequest($api, $group) {
        $m = $this->request['method'];
        $r = explode('/', trim($this->request['resource'], '/'));

        $answer = false;

        if ($m === 'GET') {
            if ($r[0] === 'players') {
                $answer = $api->getPlayers($group);
            } else if ($r[0] === 'config') {
                array_shift($r);
                $answer = $api->getConfig($group, implode('/', $r));
            } else if ($r[0] === 'tokens') {
                $answer = $api->getTokens($group);
            }
        } else if ($m === 'POST') {
            if ($r[0] === 'token') {
                $answer = $api->newToken($group, $r[1]);
            } else if ($r[0] === 'executeingamecommand') {
                $answer = $api->executeIngameCommand($group, $this->body);
            }
        } else if ($m === 'DELETE') {
            if ($r[0] === 'user') {
                $answer = $api->deleteUser($group, $r[1]);
            } else if ($r[0] === 'token') {
                $answer = $api->deleteToken($group, $r[1]);
            }
        } else if ($m === 'PUT') {
            if ($r[0] === 'config') {
                array_shift($r);
                $answer = $api->writeConfig($group, implode('/', $r), $this->body);
            } else if ($r[0] === 'user') {
                $answer = $api->addUser($group, $this->body);
            }
        }

        $this->writeToSocket($this->generateResponse(json_encode($answer)));
    }

    private function readRequestLine(&$offset) {
        $linebreak_position = strpos($this->in_buffer, "\r\n", $offset);
        if ($linebreak_position === false) {
            return false;
        }

        $first_line = substr($this->in_buffer, 0, $linebreak_position);
        $parts = explode(' ', $first_line);
        if (count($parts) !== 3) {
            return false;
        }

        $method = $parts[0];
        if (!in_array($method, ['GET', 'POST', 'PUT', 'DELETE', 'OPTIONS'], true)) {
            return false;
        }

        $resource = $parts[1];

        $protocol = $parts[2];
        if (!in_array($protocol, ['HTTP/1.1'], true)) {
            return false;
        }

        $offset = $linebreak_position + 2;

        return array('method' => $method, 'resource' => $resource, 'protocol' => $protocol);
    }

    private function readHeaderFields(&$offset) {
        $double_linebreak_position = strpos($this->in_buffer, "\r\n\r\n", $offset);
        if ($double_linebreak_position === false) {
            return false;
        }

        $header_lines = substr($this->in_buffer, $offset, $double_linebreak_position - $offset);

        $header_fields = explode("\r\n", $header_lines);

        $header = [];
        $header_fields_count = count($header_fields);
        $header_field_parts = [''];
        for ($i = 0; $i < $header_fields_count; $i++) {
            if ($header_fields[$i][0] === ' ' || $header_fields[$i][0] === "\t") {
                if ($i === 0) {
                    return false;
                }
                $header[strtolower($header_field_parts[0])] .= substr($header_fields[$i], 1);
            } else {
                $header_field_parts = explode(': ', $header_fields[$i], 2);
                if (count($header_field_parts) !== 2) {
                    return false;
                }
                $header[strtolower($header_field_parts[0])] = $header_field_parts[1];
            }
        }

        $offset = $double_linebreak_position + 4;

        return $header;
    }

    private function readBody($offset) {
        $length = (int) $this->header['content-length'];

        if ($length === 0) {
            return '';
        }

        $body = substr($this->in_buffer, $offset, $length);

        if (strlen($body) !== $length) {
            return false;
        }

        return $body;
    }
} 