<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
**/

namespace MAM\Daemon\Api;

use MAM\Daemon\Core\Registry;
use MAM\Daemon\Libraries\Logging\LogInterface;

class TcpServer {
    /**
     * @var resource
     */
    protected $server_socket;
    /**
     * @var resource[]
     */
    protected $sockets = [];
    /**
     * @var TcpClient[]
     */
    protected $clients = [];
    /**
     * @var LogInterface
     */
    protected $logging;

    /**
     * Initiates a new tcp server.
     * @param int $port listen port
     * @throws \Exception if any error occured (errors at starting socket or setting blocking mode)
     */
    public function __construct($port) {
        $this->logging = Registry::getInstance()->logging;

        $this->server_socket = stream_socket_server('tcp://0.0.0.0:' . $port, $errno, $errstr, STREAM_SERVER_BIND | STREAM_SERVER_LISTEN);
        $this->sockets[] = $this->server_socket;

        if ($this->server_socket === false) {
            $error_message = 'API TCP server: starting the server socket occured an error: (' . $errno . ') ' . $errstr;
            $this->logging->warning($error_message);
            throw new \Exception($error_message);
        }

        if (!stream_set_blocking($this->server_socket, 0)) {
            $error_message = 'API TCP server: set blocking to 0 failed';
            $this->logging->warning($error_message);
            throw new \Exception($error_message);
        }

        if (!stream_set_timeout($this->server_socket, 0)) {
            $error_message = 'API TCP server: set timeout to 0 failed';
            $this->logging->warning($error_message);
            throw new \Exception($error_message);
        }
    }

    /**
     * Accepts new client connections and writes the incoming data into the clients incoming buffers.
     */
    public function doWork() {
        $read = $this->sockets;
        $null = null;
        $ret = stream_select($read, $null, $null, 0);

        if ($ret === false) {
            $this->logging->warning('API TCP server: stream select error');
            return;
        }

        foreach ($read as $read_socket) {
            if ($read_socket === $this->server_socket) {
                while (($new_socket = @stream_socket_accept($this->server_socket, 0)) !== false) {
                    $error = false;
                    if (!stream_set_blocking($new_socket, 0)) {
                        $this->logging->warning('API TCP server: set blocking to 0 for new client socket failed');
                        $error = true;
                    }
                    if (!stream_set_timeout($new_socket, 0)) {
                        $this->logging->warning('API TCP server: set timeout to 0 for new client socket failed');
                        $error = true;
                    }
                    if ($error === true) {
                        $this->logging->warning('API TCP server: because of error(s), the client connection is dropped');
                        stream_socket_shutdown($new_socket, STREAM_SHUT_RDWR);
                        $this->onSocketDisconnect($new_socket);
                    } else {
                        $this->logging->debug('API TCP server: new client connection: ' . stream_socket_get_name($new_socket, true));
                        $this->sockets[$this->getSocketHash($new_socket)] = $new_socket;
                        $this->onSocketConnect($new_socket);
                    }
                }
            } else {
                do {
                    $new_data = fread($read_socket, 8192);
                    if (is_string($new_data) && strlen($new_data) > 0) {
                        $this->logging->debug('API TCP server: new data from client ' . stream_socket_get_name($read_socket, true) . ': ' . $new_data);
                        $this->clients[$this->getSocketHash($read_socket)]->appendInBuffer($new_data);
                    } else {
                        $this->logging->debug('API TCP server: no new data from client ' . stream_socket_get_name($read_socket, true));
                    }
                } while (!empty($new_data));
                if (feof($read_socket) === true) {
                    $this->logging->debug('API TCP server: detect client disconnect (foef)');
                    $this->onSocketDisconnect($read_socket);
                }
            }
        }
    }

    protected function onSocketConnect($socket) {
        $this->clients[$this->getSocketHash($socket)] = new TcpClient($socket, $this);
    }

    protected function onSocketDisconnect($socket) {
        unset($this->clients[$this->getSocketHash($socket)]);
        unset($this->sockets[$this->getSocketHash($socket)]);
    }

    public function getSocketHash($socket) {
        return stream_socket_get_name($socket, true);
    }

    public function getClientSockets() {
        $client_sockets = [];
        foreach ($this->sockets as $socket) {
            if ($socket !== $this->server_socket) {
                $client_sockets[] = $socket;
            }
        }
        return $client_sockets;
    }

    public function removeClient($client) {
        $this->onSocketDisconnect($client->getSocket());
    }
}
