<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
 * */

namespace MAM\Daemon\Core;

/**
 * This class HearBeat sends the Statistic to the ManuAdminMod Master Server.
 *
 * @author Mirko911 <mirko911@manuadminmod.de>
 */
class HeartBeat extends Utils {
    /**
     * Sets a value if a new Version is available
     *
     * @var boolean
     */
    private $new_version_available = false;
    /**
     * Ask all information like Plugins etc. from the MAM
     *
     * @var boolean
     */
    private $init = true;

    /**
     * Constructor
     */
    public function __construct() {
        parent::__construct();
        parent::initPlugin();

        $this->mod->setDefaultCV('daemon', 'updatecheck', 1);
        $this->mod->setDefaultCV('daemon', 'serverlist', true);

        $this->mod->registerEvent('mamStarted', 'sendHeartBeat', $this);

        $this->checkVersion();

        $this->registry->jobs->addCronJob(6, 'h', -1, array($this, 'checkversion'));
        $this->registry->jobs->addCronJob(15, 'm', -1, array($this, 'sendHeartBeat'), array(false));
    }

    /**
     * Returns a string if object is printed
     *
     * Notice: all plugins which where moved to core needs a __toString function
     * to get it working with the permissions system. Therefore the return of __toString must me the
     * namespace like daemon, punishment, permissions ...
     *
     * @see http://php.net/manual/en/language.oop5.magic.php#object.tostring
     * @return string
     */
    public function __toString() {
        return 'Heartbeat';
    }

    /**
     * Collects anonymous Basic System Infos
     *
     * The ip:port is hashed, so heartbeat is anonymous
     *
     * @param boolean $init
     * @return array anonymous heartbeat data
     */
    public function collectInfos($init = false) {
        $os = @(strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') ? 'Windows' : 'Linux';
        $phpverion = @phpversion();
        $players = count($this->players);
        $game = $this->mod->getGame();
        $lang = $this->mod->getCV('daemon', 'language');
        $serverlist = ($this->mod->getCV('daemon', 'serverlist')) ? true : false;
        $version = $this->registry->version;

        try {
            $geoip_data = $this->geoip->getLocationData($this->mod->getCV('connection', 'ip'));

            if (isset($geoip_data['country']['iso_code'])) {
                $location = $geoip_data['country']['iso_code'];
            } else {
                $location = $this->mod->getLngString('xy');
            }
        } catch (\MAM\Daemon\Libraries\GeoIP\GeoIpException $e) {
            $this->logging->notice('Geoip: ' . $e);
            $location = 'xy';
        }

        $plugins = array();
        $plugin_namespaces = $this->pluginloader->getPlugins();
        foreach ($plugin_namespaces as $plugin_namespace) {
            $plugins[$plugin_namespace] = $this->pluginloader->getPluginData($plugin_namespace);
        }

        $geodata = array();

        /**
         * @todo Disabled for bf 4 due bf4 bug
         */
        if ($this->mod->getGame() !== "bf4") {

            foreach ($this->players as $player) {
                try {
                    $geoip_data = $this->geoip->getLocationData($player->getIP());

                    if (isset($geoip_data['country']['iso_code'])) {
                        $country_code = $geoip_data['country']['iso_code'];
                    } else {
                        $country_code = $this->mod->getLngString('xy');
                    }
                } catch (\MAM\Daemon\Libraries\GeoIP\GeoIpException $e) {
                    $this->logging->notice('Geoip: ' . $e);
                    $country_code = 'xy';
                }

                if (!isset($geodata[$country_code])) {
                    $geodata[$country_code] = 1;
                } else {
                    $geodata[$country_code] ++;
                }
            }
        }

        if ($init) {
            $returnarray = array(
                'distribution' => $os,
                'php_version' => $phpverion,
                'ip' => $this->mod->getCV('connection', 'ip'),
                'port' => $this->mod->getCV('connection', 'port'),
                'location' => $location,
                'game' => $game,
                'version' => $version,
                'plugins' => $plugins,
                'language' => $lang,
                'serverlist' => $serverlist,
            );
        } else {
            $returnarray = array(
                'ip' => $this->mod->getCV('connection', 'ip'),
                'port' => $this->mod->getCV('connection', 'port'),
                'players' => $players,
                'geodata' => $geodata,
                'gametype' => $this->mod->getGameType(),
                'map' => $this->mod->getLastMaps(1)[0]
            );
        }
        return $returnarray;
    }

    /**
     * Sends the heartbeat to Manuadminmod
     *
     * @return void
     */
    public function sendHeartBeat() {
        $postdata_array = array();

        if ($this->init) {
            $postdata_array['init'] = gzcompress(json_encode($this->collectInfos(true)), 1);
            $this->init = false;
        }
        $postdata_array['h'] = gzcompress(json_encode($this->collectInfos()), 1);


        $url = 'http://www.manuadminmod.de/report.php';
        $postdata = http_build_query($postdata_array);
        $opts = array('http' =>
            array(
                'method' => 'POST',
                'header' => 'Content-type: application/x-www-form-urlencoded',
                'content' => $postdata
            )
        );
        $context = stream_context_create($opts);

        if (!\MAM\Daemon\Libraries\Helper\Helper::checkSiteAvailable($url)) {
            $this->logging->notice('Can not connect to ManuAdminMod Master Server at \'manuadminmod.de/heartbeat.php\' => Skip reporting!');
            return;
        } elseif (DEBUG) {
            $this->logging->notice('Skipped report to ManuAdminMod Master Server due Debug Mode');
            return;
        }
        $this->logging->info('Heartbeat has been sent to the ManuAdminMod Master Server: ' . file_get_contents($url, false, $context));
    }

    /**
     * Checks if a new MAM Version exists
     * @todo nested blocks in line 225
     */
    public function checkVersion() {
        if (!\MAM\Daemon\Libraries\Helper\Helper::checkSiteAvailable('http://www.manuadminmod.de/version.txt')) {
            $this->logging->notice('Can not connect to ManuAdminMod Master Server at \'manuadminmod.de/heartbeat.php\' => Skip reporting!');
            return;
        }

        $currentversionfile = file_get_contents('http://www.manuadminmod.de/version.txt');

        $local = preg_match('/(\d+).(\d+).(\d+)(-(\w+)\+(\d+))?/', $this->registry->version, $matches_local);
        preg_match('/(\d+).(\d+).(\d+)(-(\w+)\+(\d+))?/', $currentversionfile, $matches_online);

        unset($matches_local[0]);
        if (isset($matches_local[4])) {
            unset($matches_local[4]);
        }
        $new = false;
        $builds = array('DEV' => 0, 'Alpha' => 1, 'Beta' => 2, 'RC' => 3);
        if ($local > 0) {
            foreach ($matches_local as $key => $parts) {
                if (is_numeric($parts)) {
                    if ($parts < $matches_online[$key]) {
                        $new = true;
                        break;
                    }
                } else {
                    if (isset($matches_online[4])) {
                        if ($builds[$parts] < $builds[$matches_online[$key]]) {
                            $new = true;
                            break;
                        }
                    } else {
                        $new = true;
                        break;
                    }
                }
            }
        }
        if ($new || $local == 0) {
            $this->new_version_available = $currentversionfile;
            $this->announceNewVersion();
        }
    }

    /**
     * Announces if a new version available
     */
    public function announceNewVersion() {
        $this->logging->info('New version of ManuAdminMod available (' . $this->new_version_available . ') => Please update by visiting manuadminmod.de!');
        $this->rcon->rconSay($this->mod->getLngString('new_version_available', array('<VERSION>'), array($this->new_version_available)));

        $this->registry->jobs->addSingleJob($this->mod->getCV('daemon', 'announceinterval') * 60, array($this, 'announceNewVersion'));
    }
}
