<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
**/

namespace MAM\Daemon\Core;

/**
 * Registry Class
 *
 * allows global access to variables and objects
 *
 * @author Mirko911 <mirko911@manuadminmod.de>
 * @version 1.0.0
 */
class Registry {
    /**
     * Storage Array
     *
     * All global stuff is stored there
     *
     * @var array
     */
    private $storage = array();
    /**
     * Extra information to check if saved variable is constant
     *
     * @var array
     */
    private $is_constant = array();
    /**
     * Stores itself as object
     *
     * @var Registry
     */
    public static $instance = NULL;

    /**
     * Returns instance of registry
     *
     * @see http://en.wikipedia.org/wiki/Singleton_pattern
     * @return \MAM\Daemon\Core\Registry
     */
    public static function getInstance() {
        if (is_null(static::$instance)) {
            static::$instance = new self();
        }
        return static::$instance;
    }

    /**
     * Returns a string if object is printed
     *
     * Notice: all plugins which where moved to core needs a __toString function
     * to get it working with the permissions system. Therefore the return of __toString must me the
     * namespace like deamon, punishment, permissions ...
     *
     * @see http://php.net/manual/en/language.oop5.magic.php#object.tostring
     * @return string
     */
    public function __toString() {
        return 'registry';
    }

    /**
     * Trick to avoid cloning of object
     * @see http://en.wikipedia.org/wiki/Singleton_pattern
     */
    public function __clone() {

    }

    /**
     * Define a global constant
     *
     * @param string $name Constant Name
     * @param mixed $value Value, can be string or int
     * @return boolean
     */
    public function define($name, $value) {
        $name = strtolower($name);

        if ($this->has($name) && array_key_exists($name, $this->is_constant)) {
            $this->storage['logging']->error('Constant ' . $name . ' already defined');
            return false;
        } else {
            $this->storage[$name] = $value;
            $this->is_constant[$name] = true;
            return true;
        }
    }

    /**
     * Setter Method
     *
     * Sets a global var
     *
     * @param string $name Name for identification
     * @param mixed $value Content can be everything
     */
    public function __set($name, $value) {
        $name = strtolower($name);

        if ($this->has($name)) {
            $this->storage['logging']->debug('Overwriting ' . $name . ' in Registry');
        }
        $this->storage[$name] = $value;
    }

    /**
     * Gets a value from storage
     *
     * @param string $name Name for identification
     * @return mixed $value Content can be everything
     */
    public function &__get($name) {
        $name = strtolower($name);

        if ($this->has($name)) {
            return $this->storage[$name];
        } else {
            $this->storage['logging']->error($name . ' doesn\'t exist in Registry');
        }
        return "";
    }

    /**
     * Check if value exists in storage
     *
     * @param string $name
     * @return boolean True if isset, false if not
     */
    public function has($name) {
        $name = strtolower($name);

        if (array_key_exists($name, $this->storage)) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Remove value from storage
     *
     * @param string $name Name for identification
     * @return boolean True if successful, false if not
     */
    public function remove($name) {
        $name = strtolower($name);

        if ($this->has($name) && !array_key_exists($name, $this->is_constant)) {
            unset($this->storage[$name]);
            return true;
        } elseif ($this->has($name) && array_key_exists($name, $this->is_constant)) {
            $this->storage['logging']->warning('Can\'t remove ' . $name . ' because it\'s a constant');
            return false;
        } else {
            $this->storage['logging']->debug('Can\'t remove ' . $name . ' because it doesn\'t exists');
            return true;
        }
    }

    /**
     * Returns the whole storage array
     *
     * @return array
     */
    public function getAll() {
        return $this->storage;
    }
}
