<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
 * */

namespace MAM\Daemon\Libraries\Helper;

/**
 * Helper Class
 *
 * This class contains functions which just "helps"
 * Means that this functions only contains code snippets which makes something a bit easier
 *
 * @author Mirko911 <mirko911@manuadminmod.de>
 */
class Helper {

    /**
     * Returns the engine of a game
     *
     * @param string $game Name of the game
     * @return string|null
     */
    public static function getGameEngine($game) {
        $game_engines = array(
            "cod2" => "quake3",
            "cod4" => "quake3",
            "cod5" => "quake3",
            "bf4" => "frostbite3"
        );
        if (array_key_exists($game, $game_engines)) {
            return $game_engines[$game];
        } else {
            return NULL;
        }
    }

    /**
     * Parse Staratup Arguments
     *
     * @global array $argv
     * @global array $argc
     * @param array $possible
     * @return array
     * @throws \Exception
     */
    public static function parseArgv($possible) {
        global $argc, $argv;
        if (empty($argc)) {
            throw new \Exception("No arguments passed");
        }
        $parsed = array();

        for ($i = 1; $i < $argc; $i ++) {
            if ($argv[$i][0] != "-" && $argv[$i][0] != "/") {
                throw new \Exception("Unrecognized part '$argv[$i]'");
            }

            $name = strtolower(substr($argv[$i], 1));
            if (!array_key_exists($name, $possible)) {
                throw new \Exception("Unrecognized parameter '$argv[$i]'");
            }

            if (array_key_exists($name, $parsed) && !$possible[$name]["multiple"]) {
                throw new \Exception("'$name' can't be set multiple times");
            }

            $values = array();
            for ($j = 1; $j <= $possible[$name]["count"]; $j++, $i++) {

                if (!isset($argv[$i + 1])) {
                    throw new \Exception("Wrong parameter count for '$name'");
                }

                if ($possible[$name]["count"] == 1) {
                    $values = $argv[$i + 1];
                } else {
                    $values[] = $argv[$i + 1];
                }
            }

            if ($possible[$name]["multiple"]) {
                if (!array_key_exists($name, $parsed)) {
                    $parsed[$name] = array();
                }
                $parsed[$name][] = $values;
            } else {
                $parsed[$name] = $values;
            }
        }
        return $parsed;
    }

    /**
     * Converts seconds in a better readable format
     *
     * This function can be used to generate uptime from seconds.
     * You can use the default output format '%a <DAYS>, %h <HOURS>, %i <MINUTES> <AND> %s <SECONDS>'
     * or you use your own one as parameter
     *
     * You can also use the following pattern to format the string into the language which is defined in config.json
     * <ul>
     * <li><MONTHS => %m></li>
     * <li><DAYS> => %d</li>
     * <li><HOURS> => %h</li>
     * <li><MINUTES> => %i</li>
     * <li><SECONDS> => %s</li>
     * <li><AND></li>
     * </ul>
     *
     * @param int $seconds positive Integer
     * @param string|false $format_string String to format the output. [OPTIONAL] Default: '%a <DAYS>, %h <HOURS>, %i <MINUTES> <AND> %s <SECONDS>'
     * @return string
     */
    public static function convertSeconds($seconds, $format_string = false) {
        if (!is_numeric($seconds) || $seconds < 0) {
            $output = '0 <DAYS>, 0 <HOURS>, 0 <MINUTES> <AND> 0 <SECONDS>';
        } else {
            $dtF = new \DateTime("@0");
            $dtT = new \DateTime("@$seconds");
            if ($format_string != false) {
                $output = $dtF->diff($dtT)->format($format_string);
            } else {
                $output = $dtF->diff($dtT)->format('%a <DAYS>, %h <HOURS>, %i <MINUTES> <AND> %s <SECONDS>');
            }
        }

        /* @var $mod \MAM\Daemon\Core\Mod */
        $mod = \MAM\Daemon\Core\Registry::getInstance()->mod;
        $search = array(
            '<MONTHS>',
            '<DAYS>',
            '<HOURS>',
            '<MINUTES>',
            '<SECONDS>',
            '<AND>',
        );
        $replace = array(
            $mod->getLngString('months'),
            $mod->getLngString('days'),
            $mod->getLngString('hours'),
            $mod->getLngString('minutes'),
            $mod->getLngString('seconds'),
            $mod->getLngString('and'),
        );

        return str_replace($search, $replace, $output);
    }

    /**
     * Generates the uptime with seconds
     * @param int $time
     * @return String
     */
    public static function makeUptime2($time) {        //Generiert aus Sekunden  => Stunden, Minuten und Sekunden
        $sekunden = $time % 60;
        $weiter = ($time - $sekunden) / 60;
        $minuten = $weiter % 60;
        $stunden = ($weiter - $minuten) / 60;
        return sprintf("%02d:%02d:%02d", (int) $stunden, (int) $minuten, (int) $sekunden);
    }

    /**
     * Outputs the Filesize
     * @param int $size
     * @return string
     */
    public static function size($size) {
        $sizes = Array('Bytes', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB');
        $ext = $sizes[0];
        $sizecount = count($sizes);
        for ($i = 1; (($i < $sizecount) && ($size >= 1024)); $i++) {
            $size = $size / 1024;
            $ext = $sizes[$i];
        }
        $size = round($size, 2) . ' ' . $ext;
        return $size;
    }

    /**
     * Parses a Json file and throws exception if error occurred
     *
     * @param  string $file file you want to parse
     * @param boolean $lowercase_mapping All parsed array keys will be lowercase if enabled
     * @throws \Exception json error
     * @return array representation of the json file
     */
    public static function parseJson($file, $lowercase_mapping = true) {
        $log = \MAM\Daemon\Core\Registry::getInstance()->logging;

        if (!file_exists($file)) {
            $log->warning("'$file' does not exist.");
            return array();
        }

        $obj = json_decode(file_get_contents($file), true);

        $error = json_last_error();
        if ($error !== JSON_ERROR_NONE) {
            if (function_exists('json_last_error_msg')) {
                $log->error("Error in file '$file': " . json_last_error_msg());
            }

            $errors = array(
                JSON_ERROR_NONE => null,
                JSON_ERROR_DEPTH => 'Maximum stack depth exceeded',
                JSON_ERROR_STATE_MISMATCH => 'Underflow or the modes mismatch',
                JSON_ERROR_CTRL_CHAR => 'Unexpected control character found',
                JSON_ERROR_SYNTAX => 'Syntax error, malformed JSON',
                JSON_ERROR_UTF8 => 'Malformed UTF-8 characters, possibly incorrectly encoded'
            );

            $log->error("Error in file '$file': " . array_key_exists($error, $errors) ? $errors[$error] : "Unknown error ({$error})");
        }

        return (is_bool($lowercase_mapping) && $lowercase_mapping) ? static::arrayChangeKeyCaseRecursive($obj) : $obj;
    }

    /**
     * Recursively changes the case of all keys in an array
     *
     * Returns an array with all keys from <b>input</b> lowercased or uppercased. Numbered indices are left as is.
     *
     * @param  array $input  The array to work on
     * @param  int   $case   Either <b>CASE_UPPER</b> or <b>CASE_LOWER</b> (default)
     * @throws \E_WARNING if <b>input</b> is not an array.
     * @return array Returns an array with its keys lower or uppercased, or <b>FALSE</b> if <b>input</b> is not an array.
     */
    public static function arrayChangeKeyCaseRecursive($input, $case = CASE_LOWER) {
        return array_map(function($item) {
            if (is_array($item)) {
                $item = static::arrayChangeKeyCaseRecursive($item);
            }
            return $item;
        }, array_change_key_case($input, $case));
    }

    /**
     * Converts a one dimensional array to a multidimensional
     *
     * array('part1','part2','part3') will be changed to
     * array('part1' =>
     *              array('part2' =>
     *                      arrayy('part3')
     *              )
     *      )
     *
     * @param array $one_dimensional_array
     * @return array multidimensional array
     */
    public static function convertArraytoMultidimensional($one_dimensional_array) {
        $array = array();
        $first = true;
        foreach (array_reverse($one_dimensional_array) as $arr) {
            if ($first) {
                $array = array($arr);
                $first = false;
                continue;
            }
            $array = array($arr => $array);
        }
        return $array;
    }

    /**
     * array_merge_recursive does indeed merge arrays, but it converts values with duplicate
     * keys to arrays rather than overwriting the value in the first array with the duplicate
     * value in the second array, as array_merge does. I.e., with array_merge_recursive,
     * this happens (documented behavior):
     *
     * array_merge_recursive(array('key' => 'org value'), array('key' => 'new value'));
     *     => array('key' => array('org value', 'new value'));
     *
     * array_merge_recursive_distinct does not change the datatypes of the values in the arrays.
     * Matching keys' values in the second array overwrite those in the first array, as is the
     * case with array_merge, i.e.:
     *
     * array_merge_recursive_distinct(array('key' => 'org value'), array('key' => 'new value'));
     *     => array('key' => array('new value'));
     *
     * Parameters are passed by reference, though only for performance reasons. They're not
     * altered by this function.
     *
     * @param array $array1
     * @param array $array2
     * @return array
     * @author Daniel <daniel (at) danielsmedegaardbuus (dot) dk>
     * @author Gabriel Sobrinho <gabriel (dot) sobrinho (at) gmail (dot) com>
     */
    public static function arrayMergeRecursiveDistinct(array &$array1, array &$array2) {
        $merged = $array1;

        foreach ($array2 as $key => &$value) {
            if (is_array($value) && isset($merged [$key]) && is_array($merged [$key])) {
                $merged [$key] = static::arrayMergeRecursiveDistinct($merged [$key], $value);
            } else {
                $merged [$key] = $value;
            }
        }

        return $merged;
    }

    /**
     * Creates the database tables in a mysql/sqlite database
     *
     * If the database doesn't exist, mam creates the tables
     * This check runs every mam start.
     * It also sets the online counter for every player to 0
     */
    public static function createTables() {
        /* @var $mod \MAM\Daemon\Core\Mod */
        $mod = \MAM\Daemon\Core\Registry::getInstance()->mod;

        /* @var $database \MAM\Daemon\Libraries\Database\Database */
        $database = \MAM\Daemon\Core\Registry::getInstance()->database;

        if ($mod->getCV('database', 'databasetype') == 'mysql') {
            $sql = file_get_contents('daemon' . DIRECTORY_SEPARATOR . 'libraries' . DIRECTORY_SEPARATOR . 'database' . DIRECTORY_SEPARATOR . 'tables_mysql.sql');
            $database->exec($sql);
        } else {
            $sql = file_get_contents('daemon' . DIRECTORY_SEPARATOR . 'libraries' . DIRECTORY_SEPARATOR . 'database' . DIRECTORY_SEPARATOR . 'tables_sqlite.sql');
            $parts = explode('#@#$', $sql);
            foreach ($parts as $value) {
                $database->exec($value);
            }
        }

        $database->query('UPDATE <PFX>user SET online = 0 WHERE online = 1');
    }

    /**
     * Creates a infobox with a border
     *
     * with this function you can create infoboxes like this
     *
     * ******************************************************
     * *                CREATED ADMIN TOKEN                 *
     * *                      -epJYQF_                      *
     * * You can use it ingame over !group usetoken <TOKEN> *
     * ******************************************************
     *
     * The text of this box is automatically centered and the width of the box
     * equals the width of the largest text part + 2 whitespaces.
     * You can also change the char of the border and the logtype
     *
     * @param array $text The text you want to send in an array. Each line needs an own array part
     * @param type $borderchar The char for your border
     * @param type $logtype The logtype (notice, warning, info, debug)
     * @return boolean true if successful, false if not
     */
    public static function createInfoBox(array $text = array(), $borderchar = "=", $logtype = 'info') {
        if (empty($text)) {
            return false;
        }

        /* @var $logging \MAM\Daemon\Libraries\Logging\Loghandler  */
        $logging = \MAM\Daemon\Core\Registry::getInstance()->logging;

        $maxlen = max(array_map('strlen', $text)) + 2;

        $logging->$logtype(str_pad('', $maxlen + 2, $borderchar));
        foreach ($text as $line) {
            $logging->$logtype($borderchar . str_pad($line, $maxlen, ' ', STR_PAD_BOTH) . $borderchar);
        }
        $logging->$logtype(str_pad('', $maxlen + 2, $borderchar));
        return true;
    }

    /**
     * Returns the hexadecimal representation of a raw byte string.
     * Every two chars there is a space for better readability.
     * @param string $input raw byte string
     * @return string
     */
    public static function rawByteStringToHexRepresentation($input) {
        return trim(chunk_split(bin2hex($input), 2, ' '));
    }

    /**
     * Checks if a site is online to avoid 404 errors
     *
     * @param String $site The URL to check
     * @return boolean True if Successful
     */
    public static function checkSiteAvailable($site) {
        $headers = @get_headers($site);
        if (stripos($headers[0], '40') !== false || stripos($headers[0], '50') !== false || stripos($headers[0], '301') !== false) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * Converts a potential unsafe path (containing upper directory paths '..') to a safe path by just ignoring the '..'s
     * @param $file_path
     * @return string
     */
    public static function makeSafeFilePath($file_path) {
        // convert \ to /
        $normalized = '/' . str_replace('\\', '/', $file_path) . '/';
        // convert /../ to /
        $safe = str_replace('/../', '/', $normalized);
        // trim /
        $trimmed = trim($safe, '/');
        return $trimmed;
    }
}
