<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
**/

namespace MAM\Daemon\Libraries\Punishment;

use MAM\Daemon\Core\Registry;

/**
 * Class for punishments
 *
 * This class contains all functions for punishments
 *
 * @author Master of Little <masteroflittle@manuadminmod.de>
 */
class Punishment {
    /**
     * Class for mam related stuff
     *
     * @var \MAM\Daemon\Engine\Quake3\Mod
     * @var \MAM\Daemon\Engine\Frostbite3\Mod
     */
    private $mod;
    /**
     * Class for Remote Commands
     *
     * @var \MAM\Daemon\Engine\Quake3\Rcon\RconCommands
     * @var \MAM\Daemon\Engine\Frostbite3\Rcon\RconCommands
     */
    private $rcon;
    /**
     * Class for player related stuff
     *
     * @var \MAM\Daemon\Core\Player
     * @var \MAM\Daemon\Engine\Frostbite3\Player
     *
     */
    private $players;
    /**
     * Class for Logging
     *
     * @var \MAM\Daemon\Libraries\Logging\LogHandler
     */
    private $logging;

    /**
     * @todo MasterOfLittle Add Docblock
     */
    private $callbacks = array();
    /**
     * @todo MasterOfLittle Add Docblock
     */
    private $scores = array();

    /**
     * Constructor for punishment class
     */
    public function __construct() {
        $registry = Registry::getInstance();
        $this->mod = $registry->mod;
        $this->players = & $registry->players;
        $this->rcon = $registry->rcon;
        $this->logging = $registry->logging;

        $this->mod->registerEvent('playerJoined', 'playerJoin', $this);
        $this->mod->registerEvent('playerQuit', 'playerQuit', $this);
        $this->mod->registerEvent('mamStarted', 'mamStarted', $this);

        $this->db = new Database();

        new BanHelper();
    }

    /**
     * This function is called when the mam is started / done with plugin loading
     *
     * It creates a db entry for every user
     */
    public function mamStarted() {
        foreach ($this->players as $guid => $player) {
            if (!$this->db) {
                $this->players[$guid]->setPunishmentScore(0);
            } else {
                $this->db->createUser($guid);
                $this->players[$guid]->setPunishmentScore($this->db->getScore($guid));
            }
        }
    }

    /**
     * Creates an player in the db if enabled and adds the score
     * @param $guid
     */
    public function playerJoin($guid) {
        if (!$this->db) {
            $this->players[$guid]->setPunishmentScore(0);
        } else {
            $this->db->createUser($guid);
            $this->players[$guid]->setPunishmentScore($this->db->getScore($guid));
        }
    }

    /**
     * saves the score to the db
     * @param $guid
     */
    public function playerQuit($guid) {
        if ($this->db) {
            $this->db->saveScore($guid);
        }
    }

    /**
     * Returns a string if object is printed
     *
     * Notice: all plugins which where moved to core needs a __tostring function
     * to get it working with the permissions system. Therefore the return of __tostring must me the
     * namespace like deamon, punishment, permissions ...
     *
     * @see http://php.net/manual/en/language.oop5.magic.php#object.tostring
     * @return string
     */
    public function __toString() {
        return "Punishment class";
    }

    /**
     * parse punishments from cfg-files
     */
    public function parseCfg() {
        $actions = $this->mod->getCV("punishment", "actions");
        foreach ($actions as $action) {
            $this->addPunishment($action[0], $action[1], $action[2]);
        }
    }

    public function crossed($score, $test, $added) {
        if ($test - $added < $score && $test >= $score) {
            return true;
        }
    }

    /**
     * Adds a punishment for $score of $type with standard $args
     * @param string $score
     * @param string $type
     * @param array $args
     */
    public function addPunishment($score, $type, $args) {
        if (array_key_exists($type, $this->callbacks)) {
            if (!array_key_exists($score, $this->scores)) {
                $this->scores[$score]["type"] = $type;
                $this->scores[$score]["args"] = $args;
                ksort($this->scores);
            } else {
                $this->logging->error("Punishment at $score does already exist");
            }
        } else {
            $this->logging->warning("Punishment $type does not exist. Skipping.");
        }
    }

    /**
     * Checks if the score has reached a punishment level
     *
     * @param   int         $test score to test
     * @param   int         $added added score
     * @return  bool|array  array if a punishment can be executed
     */
    public function checkScore($test, $added) {
        if ($test === 0) {
            return false;
        }
        $action = 0;
        foreach ($this->scores as $score => $type) {
            if ($this->crossed($score, $test, $added)) {
                $action = $score;
            }
            if ($test < $score) {
                break;
            }
        }
        return $this->scores[$action];
    }

    /**
     * Register a callback for a punishment. The function has to be in the player-class
     * @param string $name
     * @param string $callback
     * @param array $args standard arguments like array("time" => 5)
     * @return bool
     * @todo callbacks not only from player.class
     */
    public function registerPunishment($name, $callback, $args) {
        $game = '\MAM\Daemon\Engine\\' . ucfirst($this->mod->getEngine()) . '\\Player';
        if (is_callable(array($game, $callback)) !== false) {
            $this->callbacks[$name]["callback"] = $callback;
            $this->callbacks[$name]["args"] = $args;
            return true;
        } else {
            $this->logging->error("Callback $callback not valid");
            return false;
        }
    }

    /**
     * Executes a punishment
     * @param string $guid
     * @param array $type like array("type" => "tempban", "args" => array("time" => 5))
     * @param int $score_added added Score
     * @param array $args arguments will be merged with standard args
     * @return bool|mixed
     */
    public function punish($guid, $type, $score_added, $args = array()) {
        $call_args = array();
        if (array_key_exists($type["type"], $this->callbacks)) {
            $args += $type["args"];
            foreach ($this->callbacks[$type["type"]]["args"] as $key => $name) {
                if (!array_key_exists($name, $args)) {
                    $this->logging->error("Argument $name not declared for " . $type["type"]);
                }
                $call_args[$key] = $args[$name];
            }
            call_user_func_array(array($this->players[$guid], $this->callbacks[$type["type"]]["callback"]), $call_args);
            $this->players[$guid]->setLastPunishment($type);
            /*
              if ($this->db) {
              $this->db->logScore($guid, $args["reason"], $score_added);
              }
             */
        } else {
            $this->logging->error("Called unknown punishment: " . $type["type"]);
            return false;
        }
    }
}
