<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
 * */

namespace MAM\Daemon\Libraries\TeamSpeak3;

use \MAM\Daemon\Core\Registry;

/**
 * Library for interacting with a TeamSpeak3 Server Query socket.
 *
 * @author Jens Forstmann
 * @link http://media.teamspeak.com/ts3_literature/TeamSpeak%203%20Server%20Query%20Manual.pdf
 */
class ServerQuery {
    /**
     * @var resource
     */
    private $socket;
    /**
     * Chars which aren't allowed in TS3 server query.
     * @var array
     */
    private $chars;
    /**
     * Escape chars to translate the forbidden chars into TS3 server query format.
     * @var array
     */
    private $escape;
    /**
     * @var \MAM\Daemon\Libraries\Logging\LogHandler
     */
    protected $logging;
    /**
     * @var boolean
     */
    public $log_errors = false;
    /**
     * Commands which will not be executed.
     * @var array
     */
    private $command_blacklist = array('', 'help');
    /**
     * @var callable[]
     */
    private $on_notify_text_message_callable = array();

    /**
     * Initiates a ServerQuery object and connects to the TeamSpeak3 server.
     * @param string $ip
     * @param int $port
     * @throws \Exception if connection fails
     */
    public function __construct($ip, $port = 10011) {
        $this->logging = Registry::getInstance()->logging;
        $this->chars = array(chr(92), chr(47), chr(32), chr(124), chr(7), chr(8), chr(12), chr(10), chr(13), chr(9), chr(11));
        $this->escape = array(chr(92) . chr(92), chr(92) . chr(47), chr(92) . chr(115), chr(92) . chr(112), chr(92) . chr(97), chr(92) . chr(98), chr(92) . chr(102), chr(92) . chr(110), chr(92) . chr(114), chr(92) . chr(116), chr(92) . chr(118));

        $this->connect($ip, $port);

        if (trim(fgets($this->socket)) !== 'TS3') {
            $error_message = 'TeamSpeak3: Not connected to a valid server query';
            $this->logging->warning($error_message);
            $this->disconnect();
            throw new \Exception($error_message);
        }

        /* @var $jobs \MAM\Daemon\Libraries\Jobs\Jobs */
        $jobs = Registry::getInstance()->jobs;
        $jobs->addEveryTimeJob(array($this, 'cron'));
        $jobs->addCronJob(1, 'm', -1, array($this, 'keepAlive'));
    }

    /**
     * Destructs this object.
     */
    public function __destruct() {
        $this->disconnect();
    }

    /**
     * Converts a string from PHP format into TS format.
     * @param string $string
     * @return string
     */
    public function TS3String($string) {
        return str_replace($this->chars, $this->escape, $string);
    }

    /**
     * Converts a string from TS format into PHP format.
     * @param string $string
     * @return string
     */
    protected function PHPString($string) {
        return str_replace($this->escape, $this->chars, $string);
    }

    /**
     * Connects to the server query socket
     * @param string $ip
     * @param int $port
     * @return boolean true on success
     * @throws \Exception if connection fails
     */
    private function connect($ip, $port) {
        // try to connect to socket
        $s = @stream_socket_client('tcp://' . $ip . ':' . $port, $errno, $errstr);
        if ($s === false) {
            $error_message = 'TeamSpeak3: Connection failed: ' . trim($errstr) . ' (' . $errno . ')';
            $this->logging->warning($error_message);
            throw new \Exception($error_message);
        }
        stream_set_timeout($s, 1);

        // try to disconnect from old socket
        $this->disconnect();

        $this->socket = $s;

        $this->logging->info('TeamSpeak3: Connected to ' . $ip . ':' . $port);

        return true;
    }

    /**
     * Disconnects from the server query socket
     * @throws \Exception if socket shutdown fails
     */
    private function disconnect() {
        if (is_resource($this->socket)) {
            $this->logging->debug('TeamSpeak3: Disconnect');
            if (!stream_socket_shutdown($this->socket, STREAM_SHUT_RDWR)) {
                $error_message = 'TeamSpeak3: Disconnecting causes an error';
                $this->logging->warning($error_message);
                throw new \Exception($error_message);
            }
        }
    }

    /**
     * Executes a command and returns the answer in array format.
     * (Check isError method for errors.)
     * @param string $command the server query command, use TS3String() method to escape strings properly
     * @param bool $exception_on_error if true an exception will be thrown if an error occurs
     * @return array entry 'answer' containing elements for each item and an 'error' entry with the returned status code
     * @throws \Exception only if an error occurs and $exception_on_error is true
     */
    public function exec($command, $exception_on_error = false) {
        $command = trim($command);
        if (in_array($command, $this->command_blacklist)) {
            $this->logging->debug('TeamSpeak3: Executing command "' . $command . '" is not allowd. Skip.');
            return array();
        }
        $this->logging->debug('Teamspeak3: Execute command: ' . $command);
        fwrite($this->socket, $command . "\n");
        $answer = $this->decodeAnswer();
        if ($this->isError($answer)) {
            if ($this->log_errors) {
                $this->logging->info('Teamspeak3: Command "' . $command . '" caused an error: (' . $answer['error']['id'] . ') ' . $answer['error']['msg']);
            }
            if ($exception_on_error) {
                throw new \Exception($answer['error']['msg']);
            }
        }
        return $answer;
    }

    /**
     * Checks if the answer contains an error.
     * @param array $answer
     * @return bool
     */
    public function isError($answer) {
        return $answer['error']['id'] !== '0';
    }

    /**
     * Checks if the answer contains no error.
     * @param array $answer
     * @return bool
     */
    public function isOk($answer) {
        return $answer['error']['id'] === '0';
    }

    /**
     * Will be called every second to receive and process anything.
     */
    public function cron() {
        if (is_resource($this->socket)) {
            stream_set_blocking($this->socket, 0);
            $this->decodeAnswer(true);
            stream_set_blocking($this->socket, 1);
        }
    }

    /**
     * Sends empty line to keep alive.
     */
    public function keepAlive() {
        if (is_resource($this->socket)) {
            fwrite($this->socket, "\n");
        }
    }

    /**
     * Receives the ingoing traffic and transforms the data into an array.
     * @param bool if true method will return immediately if no data is received
     * @return array
     */
    public function decodeAnswer($cron = false) {
        $answer = array();
        $max_iterations = 10;
        for ($i = 0; $i < $max_iterations; $i++) {
            $received = fgets($this->socket);
            if ($received === false && $cron === true) {
                return array();
            }
            $line = trim($received); // trim is important for removing the \r at the beginning of the line
            if ($received !== false) {
                $answer = $this->decodeLine($line, $answer);
            }
            if (isset($answer['error'])) {
                break;
            }
        }
        if ($i === $max_iterations) {
            $this->logging->warning('TeamSpeak3: decodeAnswer: too many iterations. Abort.');
            if (!isset($answer['error'])) {
                $answer['error']['id'] = '1';
            }
        }
        return $answer;
    }

    /**
     * Transforms a line.
     * @param string $line
     * @param array $answer current answer array
     * @return array $answer array with the new content of $line appended
     */
    private function decodeLine($line, array $answer) {
        if (strpos($line, 'error') === 0) {
            $answer['error'] = $this->decodecell($line);
        } else if (strpos($line, 'notifytextmessage') === 0) {
            $this->onNotifyTextMessage($this->decodeCell($line));
        } else {
            foreach (explode('|', $line) as $cell) {
                $answer['answer'][] = $this->decodeCell($cell);
            }
        }
        return $answer;
    }

    /**
     * Transforms a cell into a key/value associative array.
     * @param string $cell
     * @return array
     */
    private function decodeCell($cell) {
        $cells = array();
        foreach (explode(' ', $cell) as $pair) {
            if (strpos($pair, '=') === false) {
                $cells[$pair] = '';
            } else {
                list($key, $value) = explode('=', $pair, 2);
                $cells[$key] = $this->PHPString($value);
            }
        }
        return $cells;
    }

    /**
     * Adds a callable handler for notify text messages.
     *
     * IMPORTANT: Do not forget to register for these kind of messages with the command "servernotifyregister".
     * With "servernotifyregister event=textprivate" you will receive answers of clients after sending text messages
     * to them (so that they have a private chat tab).
     * @param $callable
     * @todo add "servernotifyregister" here, so that will be done automatically
     */
    public function addNotifyTextMessageHandler($callable) {
        if (!is_callable($callable)) {
            $this->logging->notice('TeamSpeak3: Adding notify text message handler skipped (no valid callable)');
            return;
        }

        if (in_array($callable, $this->on_notify_text_message_callable)) {
            $this->logging->notice('TeamSpeak3: Adding notify text message handler skipped (handler already exists)');
            return;
        }

        $this->on_notify_text_message_callable[] = $callable;
        $this->logging->debug('TeamSpeak3: Notify text message handler added');
    }

    /**
     * Removes a text notify handler.
     * @param $callable
     * @todo add "servernotifyunregister" here, if callable list is empty
     */
    public function removeNotifyTextMessageHandler($callable) {
        $key = array_search($callable, $this->on_notify_text_message_callable);
        if ($key === false) {
            $this->logging->notice('TeamSpeak3: Removing notify text message handler skipped (handler does not exist)');
        } else {
            unset($this->on_notify_text_message_callable[$key]);
            $this->logging->debug('TeamSpeak3: Notify text message handler removed');
        }
    }

    /**
     * Will be called if a "notifytextmessage" arrives.
     * @param array $cells
     */
    private function onNotifyTextMessage(array $cells) {
        foreach ($this->on_notify_text_message_callable as $callable) {
            if (is_callable($callable)) {
                var_dump($cells);
                call_user_func_array($callable, array($cells));
            }
        }
    }
}
