<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
 * */

namespace MAM\Daemon\Libraries\TeamSpeak3;

/**
 * Provides common functions of the TeamSpeak3 Server Query API.
 *
 * @author Jens Forstmann
 * @link http://media.teamspeak.com/ts3_literature/TeamSpeak%203%20Server%20Query%20Manual.pdf
 */
class TeamSpeak3 extends ServerQuery {

    /**
     * Initiates a TeamSpeak3 object and connects to the TeamSpeak3 server query.
     * Call useVirtualServerByPort() to interact with a specific virtual server.
     * @param string $username
     * @param string $password
     * @param string $ip
     * @param int $server_query_port TCP port of the server query
     * @param int $voice_port UDP port of the virtual voice server
     * @throws \Exception if connection or login fails
     */
    public function __construct($username, $password, $ip, $server_query_port = 10011, $voice_port = 9987) {
        parent::__construct($ip, $server_query_port);
        $this->exec('whoami'); //dummy command to empty buffers
        if (!$this->login($username, $password)) {
            $error_message = 'TeamSpeak3: login failed';
            $this->logging->warning($error_message);
            throw new \Exception($error_message);
        }
        if (!$this->useVirtualServerByPort($voice_port)) {
            $this->logging->warning('TeamSpeak3: Could not select virtual server by port');
        }
    }

    /**
     * Quits the connection.
     */
    public function quit() {
        $this->exec('quit');
    }

    /**
     * Authenticates as a server query user.
     * @param string $username
     * @param string $password
     * @return bool true on success, false if login fails
     */
    public function login($username, $password) {
        return $this->isOk($this->exec('login ' . $username . ' ' . $password));
    }

    /**
     * Logouts the server query user.
     * @return bool true on success, false on error
     */
    public function logout() {
        return $this->isOk($this->exec('logout'));
    }

    /**
     * Returns the complete clientlist with all details.
     * @return array|bool false on error
     */
    public function getClientlist() {
        $clientlist = $this->exec('clientlist -uid -away -voice -times -groups -info -icon -country');
        if ($this->isError($clientlist)) {
            return false;
        }
        return $clientlist['answer'];
    }

    /**
     * Returns an array of clients, which match a given field with one search value.
     *
     * Example: getClientsByField('client_nickname', array('Alice', 'Bob', 'Caesar'))
     * @param string $field_name one of: clid, cid, client_database_id, client_nickname, client_type, client_away, client_away_message, client_flag_talking, client_input_muted, client_output_muted, client_input_hardware, client_output_hardware, client_talk_power, client_is_talker, client_is_priority_speaker, client_is_recording, client_is_channel_commander, client_unique_identifier, client_servergroups, client_channel_group_id, client_channel_group_inherited_channel_id, client_version, client_platform, client_idle_time, client_created, client_lastconnected, client_icon_id, client_country
     * @param string[] $search_values what you want to search (must be a string array)
     * @return array|bool array with matching clients (with all details), flase on error
     */
    public function getClientsByField($field_name, array $search_values, $case_sensitive = true) {
        $clientlist = $this->getClientlist();
        if ($clientlist === false) {
            return false;
        }
        $clients = array();
        foreach ($clientlist as $client) {
            foreach ($search_values as $search) {
                if (isset($client[$field_name]) &&
                    (($case_sensitive === true && $client[$field_name] === $search) || (mb_strtolower($client[$field_name]) === mb_strtolower($search)))) {
                    $clients[] = $client;
                }
            }
        }
        return $clients;
    }

    /**
     * Sends a text message.
     * @param type $target_mode (1: client, 2: channel, 3: server)
     * @param int $target_id
     * @param string $msg
     * @return bool true on success, false on error
     */
    private function sendTextMessage($target_mode, $target_id, $msg) {
        return $this->isOk($this->exec('sendtextmessage targetmode=' . $target_mode . ' target=' . $target_id . ' msg=' . $this->TS3String($msg)));
    }

    /**
     * Sends a text message to a client.
     * @param int $client_id
     * @param string $msg
     * @return bool true on success, false on error
     */
    public function sendClientTextMessage($client_id, $msg) {
        return $this->sendTextMessage(1, $client_id, $msg);
    }

    /**
     * Sends a text message to a channel.
     * @param int $channel_id
     * @param string $msg
     * @return bool true on success, false on error
     */
    public function sendChannelTextMessage($channel_id, $msg) {
        return $this->sendTextMessage(2, $channel_id, $msg);
    }

    /**
     * Sends a text message to a virtual server.
     * @param int $server_id
     * @param string $msg
     * @return bool true on success, false on error
     */
    public function sendServerTextmessage($server_id, $msg) {
        return $this->sendTextMessage(3, $server_id, $msg);
    }

    /**
     * Pokes a client.
     * @param int $client_id
     * @param string $msg
     * @return bool true on success, false on error
     */
    public function clientPoke($client_id, $msg) {
        return $this->isOk($this->exec('clientpoke clid=' . $client_id . ' msg=' . $this->TS3String($msg)));
    }

    /**
     * Selects the virtual server to interact with.
     * @param int $port
     * @return bool true on success, false on error
     */
    public function useVirtualServerByPort($port = 9987) {
        return $this->isOk($this->exec('use port=' . $port));
    }
}
