<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
 * */

namespace MAM\Daemon\Libraries\Twofactorauth;

use MAM\Daemon\Core\Registry;

/**
 * Implementation of the TOTP Algorithm as specified by
 * the ietf in rfc 6238
 * @see http://tools.ietf.org/html/rfc6238
 *
 * @author belst <belstgut@manuadminmod.de>
 */
class Totp {
    protected $codelength = 6;

    /**
     * Initiates Totp object
     * @param integer $length length of the token, default = 6
     */
    public function __construct($length = 6) {
        $this->setCodelength($length);
    }

    /**
     * Creates a easy to read secret string to use for encryption
     * store this with the user information (eg: users.json)
     * users have to know this to generate the code in eg Google Authenticator
     *
     * @param  integer $length The length of the string (default = 16)
     * @return string          The secret to share with the client
     */
    public function createSecret($length = 16) {
        $secret = '';
        $valid = Base32::$map;
        unset($valid[32]);      // remove padding character =

        for ($i = 0; $i < $length; $i++) {
            $secret .= $valid[array_rand($valid)];
        }

        return $secret;
    }

    /**
     * Generates a code based on the given secret and the current time
     * @param  string $secret the secret of the user
     * @param  int    $time   the time at which to check or null for current time
     * @return string         the generated code
     */
    public function generateCode($secret, $time = null) {

        if ($time === null) {
            $time = floor(time() / 30); //hardcoded 30. sample value used in specification and also used by google authenticator
        }

        $key = Base32::decode($secret);

        // time to binary
        $time = chr(0) . chr(0) . chr(0) . chr(0) . pack('N*', $time);
        // hash with secret
        $hmac = hash_hmac('SHA1', $time, $key, true);
        $offset = ord(substr($hmac, -1)) & 0x0F;

        $value = substr($hmac, $offset, 4);

        $value = unpack('N', $value);
        $value = $value[1];
        // 32 bits only
        $value = $value & 0x7FFFFFFF; // highest positive integer in 32bit

        $mod = pow(10, $this->codelength);
        return str_pad($value % $mod, $this->codelength, '0', STR_PAD_LEFT);
    }

    /**
     * Checks, if the given code is valid using the given secret
     * checks if code is valid in time of -30s ago till 30s from now
     * @param  string  $secret The users secret
     * @param  string  $code   the code to check
     * @return boolean         true if code was/is/will be valid in -30s/now/30s from now timespan
     */
    public function checkCode($secret, $code) {
        $time = floor(time() / 30);

        for ($i = -1; $i <= 1; $i++) {
            $tmpcode = $this->generateCode($secret, $time + $i);
            if ($tmpcode === $code) {
                return true;
            }
        }

        return false;
    }

    /**
     * Sets the codelength.
     * Must be between 6 and 32 or nothing will change
     * @param  int $length the new length. Must be between 6 and 32
     * @return \MAM\Daemon\Libraries\Helper\Totp Returns $this for method chaining
     */
    public function setCodelength($length) {
        if ($length < 6 || $length > 32) {
            return $this;
        }
        $this->codelength = $length;

        return $this;
    }
}
